@@ -1,7 +1,8 @@
 import os, cgi, re
 from os.path import *
+from gettext import gettext as _
 import deskbar, deskbar.gnomedesktop
-import gtk, gtk.gdk, gnome.ui
+import gtk, gtk.gdk, gnome.ui, gobject, gnomevfs
 from htmlentitydefs import name2codepoint
 
 ICON_THEME = gtk.icon_theme_get_default()
@@ -95,5 +96,57 @@ PATH = [path for path in os.getenv("PATH").split(os.path.pathsep) if path.strip(
 def is_program_in_path(program):
 	for path in PATH:
 		prog_path = join(path, program)
-		if exists(prog_path) and isfile(prog_path):
+		if exists(prog_path) and isfile(prog_path) and os.access(prog_path, os.F_OK | os.R_OK | os.X_OK):
 			return True
+
+def spawn_async(args):
+	try:
+		gobject.spawn_async(args, flags=gobject.SPAWN_SEARCH_PATH)
+		return True
+	except Exception, e:
+		message_dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE)
+		message_dialog.set_markup("<span size='larger' weight='bold'>%s</span>\n\n '%s'" % (
+			_("Cannot execute program:"), cgi.escape(' '.join(args))))
+
+		resp = message_dialog.run()
+		if resp == gtk.RESPONSE_CLOSE:
+			message_dialog.destroy()
+
+		return False
+
+def add_to_recent(uri):
+	if hasattr(gtk, "recent_manager_get_default"):
+		# add the file to the list of recently opened files
+		manager = gtk.recent_manager_get_default()
+		manager.add_item(uri)
+
+def url_show_file(url):
+	try:
+		uri = gnomevfs.escape_host_and_path_string(url)
+		gnomevfs.url_show(uri)
+		add_to_recent(uri)
+	except Exception, e:
+		if not spawn_async([gnomevfs.get_local_path_from_uri(url)]):
+			uri = gnomevfs.escape_host_and_path_string(dirname(url))
+			url_show(uri)
+			add_to_recent(uri)
+
+def url_show(url):
+	try:
+		gnomevfs.url_show(url)
+	except Exception, e:
+		message_dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE)
+		message_dialog.set_markup("<span size='larger' weight='bold'>%s</span>\n\n '%s'" % (
+			_("Cannot show URL:"), cgi.escape(url)))
+
+		resp = message_dialog.run()
+		if resp == gtk.RESPONSE_CLOSE:
+			message_dialog.destroy()
+
+def get_proxy(deskbar):
+	if deskbar.use_proxy:
+		proxy_string = "http://%s:%d:" % (deskbar.proxy_host, deskbar.proxy_port)
+		proxies = {'http' : proxy_string}
+		return proxies
+	else:
+		return None
